import numpy
from gym import spaces
from .base import BaseOddsEnv


class BasePercentageGamblingEnv(BaseOddsEnv):
    """Base class for sports betting environments with non fixed bet size.

    Creates an OpenAI Gym environment that supports betting a non fixed amount
    on a single outcome for a single game.

    The main difference between the BaseOddsEnv is that the action space is defined
    differently (to accommodate that non fixed bet size).

    .. versionadded:: 0.2.0
    .. deprecated:: 0.4.0
        This environment will be renamed "BasePercentageOddsEnv" in 0.4.5

    Parameters
    ----------
    action_space : gym.spaces.Box of shape (2,)
        A 2-tuple, where the first index is the action itself and the second
        index is the percentage of the current balance to place on all the outcomes
        specified in the action.

    """

    def __init__(self, odds, odds_column_names, results=None):
        super().__init__(odds, odds_column_names, results)
        self.action_space = spaces.Box(low=numpy.array([0., 0.01]),
                                       high=numpy.array([self.action_space.n - 0.01, 1 / odds.shape[1]]))

    def step(self, action):
        form = int(numpy.floor(action[0]))
        self.single_bet_size = action[1] * self.balance
        return super().step(form)



import pytest
from gym.spaces import Box
import numpy


def test_attributes(basic_env):
    assert basic_env.action_space.n == 2 ** 2
    assert basic_env.observation_space == Box(low=1., high=float('Inf'), shape=(1, 2))
    assert basic_env.STARTING_BANK == 10
    assert basic_env.balance == basic_env.STARTING_BANK
    assert basic_env.current_step == 0
    assert basic_env.single_bet_size == 1


@pytest.mark.parametrize("action,expected_reward", [(0, 0), (1, -1), (2, 1), (3, 0)])
def test_step(basic_env, action, expected_reward):
    odds, reward, done, _ = basic_env.step(action)
    assert reward == expected_reward
    assert not done
    assert basic_env.current_step == 1


def test_reset(basic_env):
    odds, reward, done, _ = basic_env.step(2)
    assert reward == 1
    assert basic_env.balance == basic_env.STARTING_BANK + 1
    assert not done
    assert basic_env.current_step == 1
    odds, reward, done, _ = basic_env.step(1)
    assert reward == 2
    assert done
    basic_env.reset()
    assert basic_env.balance == basic_env.STARTING_BANK


def test_render(basic_env):
    assert basic_env.render() == "Current balance at step {}: {}".format(basic_env.current_step, basic_env.balance)


@pytest.mark.parametrize("action", range(4))
def test_step_when_balance_is_0(basic_env, action):
    basic_env.balance = 0
    odds, reward, done, _ = basic_env.step(action)
    assert reward == 0
    assert done
    assert basic_env.current_step == 0


def test_step_illegal_action(basic_env):
    basic_env.balance = 1
    odds, reward, done, _ = basic_env.step(3)  # illegal - making a double when when the balance is 1
    assert reward == -float('inf')
    assert not done
    assert basic_env.current_step == 0


@pytest.mark.parametrize("current_step_value,excpected_results", [(0, numpy.array([[0, 1]], dtype=numpy.float64)),
                                                                  (1, numpy.array([[1, 0]], dtype=numpy.float64))])
def test_get_results(basic_env, current_step_value, excpected_results):
    basic_env.current_step = current_step_value
    results = basic_env.get_results()
    assert numpy.array_equal(results, excpected_results)



from .base import BaseOddsEnv
from .base_percentage import BasePercentageGamblingEnv


class ThreeWaySoccerOddsEnv(BaseOddsEnv):
    """Environment for 3-way soccer betting with a fixed bet size."""

    def __init__(self, soccer_bets_dataframe):
        """Initializes a new environment.

        .. versionadded: 0.1.0

        Parameters
        ----------
        soccer_bets_dataframe: dataframe of shape (n_games, n_odds + 3)
            A list of games, with their betting odds, results and the teams playing.

            .. warning::

                Please make sure that the odds columns are named "home, draw, away",
                that the results column is named "result" and that the team names
                columns are named "home_team, away_team" respectively.
        """
        odds_column_names = ['home', 'draw', 'away']
        odds = soccer_bets_dataframe[odds_column_names].values
        results = soccer_bets_dataframe['result'].values if soccer_bets_dataframe['result'].notna().all() else None
        super().__init__(odds, odds_column_names, results)
        self.teams = soccer_bets_dataframe[['home_team', 'away_team']].values

    def render(self, mode='human'):  # pragma: no cover
        """Outputs the current team names, balance and step.

        Returns
        -------
        msg : str
            A string with the current team names, balance and step.
        """
        return 'Home Team {} VS Away Team {}. {}'.format(self.teams[self.current_step % self._odds.shape[0]][0],
                                                         self.teams[self.current_step % self._odds.shape[0]][1],
                                                         super().render(mode))


class ThreeWaySoccerPercentageOddsEnv(ThreeWaySoccerOddsEnv, BasePercentageGamblingEnv):
    """Environment for 3-way soccer betting with a non fixed bet size.

    .. versionadded: 0.2.0
    """
    pass



from datetime import datetime, timedelta
from pytest import fixture
from numpy import array
from pandas import DataFrame
from oddsgym.envs.base import BaseOddsEnv
from oddsgym.envs.base_percentage import BasePercentageGamblingEnv
from oddsgym.envs.soccer import ThreeWaySoccerOddsEnv, ThreeWaySoccerPercentageOddsEnv
from oddsgym.envs.daily_bets import DailyOddsEnv


@fixture()
def basic_env(request):
    return BaseOddsEnv(array([[1, 2], [3, 4]]), ['w', 'l'], [1, 0])


@fixture()
def basic_percentge_env(request):
    return BasePercentageGamblingEnv(array([[2, 1]]), ['w', 'l'], [0])


@fixture()
def three_way_env(request):
    soccer_bets_dataframe = DataFrame([{'home_team': 'FCB', 'away_team': 'PSG',
                                        'home': 1, 'draw': 2, 'away': 3, 'result': 1},
                                       {'home_team': 'MCB', 'away_team': 'MTA',
                                        'home': 4, 'draw': 3, 'away': 2, 'result': 0}])
    return ThreeWaySoccerOddsEnv(soccer_bets_dataframe)


@fixture()
def three_way_percentage_env(request):
    soccer_bets_dataframe = DataFrame([{'home_team': 'FCB', 'away_team': 'PSG',
                                        'home': 1, 'draw': 2, 'away': 3, 'result': 1},
                                       {'home_team': 'MCB', 'away_team': 'MTA',
                                        'home': 4, 'draw': 3, 'away': 2, 'result': 0}])
    return ThreeWaySoccerPercentageOddsEnv(soccer_bets_dataframe)


@fixture()
def daily_bets_env(request):
    dataframe = DataFrame([{'w': 1, 'l': 2, 'date': datetime.today().date() - timedelta(days=1), 'result': 1},
                           {'w': 4, 'l': 3, 'date': datetime.today().date() - timedelta(days=1), 'result': 0},
                           {'w': 4, 'l': 3, 'date': datetime.today().date(), 'result': 0},
                           {'w': 4, 'l': 3, 'date': datetime.today().date(), 'result': 0},
                           ])
    return DailyOddsEnv(dataframe.drop('result', 'columns'), ['w', 'l'], dataframe['result'])


@fixture()
def daily_bets_env_non_uniform(request):
    dataframe = DataFrame([{'w': 1, 'l': 2, 'date': datetime.today().date() - timedelta(days=2), 'result': 1},
                           {'w': 4, 'l': 3, 'date': datetime.today().date() - timedelta(days=1), 'result': 0},
                           {'w': 4, 'l': 3, 'date': datetime.today().date(), 'result': 0},
                           {'w': 4, 'l': 3, 'date': datetime.today().date(), 'result': 0},
                           {'w': 5, 'l': 4, 'date': datetime.today().date(), 'result': 0},
                           ])
    return DailyOddsEnv(dataframe.drop('result', 'columns'), ['w', 'l'], dataframe['result'])



import pytest
import numpy
from gym.spaces import Box


def test_attributes(basic_percentge_env):
    assert basic_percentge_env.action_space == Box(low=numpy.array([0., 0.01]),
                                                   high=numpy.array([2 ** 2 - 0.01, 0.5]))
    assert basic_percentge_env.observation_space == Box(low=1., high=float('Inf'), shape=(1, 2))
    assert basic_percentge_env.STARTING_BANK == 10
    assert basic_percentge_env.balance == basic_percentge_env.STARTING_BANK
    assert basic_percentge_env.current_step == 0
    assert basic_percentge_env.single_bet_size == 1


@pytest.mark.parametrize("form,percent,expected_reward", [(0, 0.25, 0), (1, 0.25, 2.5),
                                                          (2, 0.25, -2.5), (3, 0.25, 0)])
def test_step(basic_percentge_env, form, percent, expected_reward):
    odds, reward, done, _ = basic_percentge_env.step((form, percent))
    assert reward == expected_reward
    assert done
    assert basic_percentge_env.current_step == 0



import gym
import numpy
from more_itertools import powerset
from pandas import DataFrame


class BaseOddsEnv(gym.Env):
    """Base class for sports betting environments.

    Creates an OpenAI Gym environment that supports betting a fixed amount (1)
    on a single outcome for a single game.

    .. versionadded: 0.1.0

    Parameters
    ----------
    observation_space : gym.spaces.Box
        The observation space for the environment.
        The observation space shape is (1, N) where N is the number of possible
        outcomes for the game.

        .. versionchanged: 0.3.0
            Changed definition of space

    action_space : gym.spaces.Discrete
        The action space for the environment.
        The action space is a single number in [0, 2 ** N), represneting
        on what outcomes to place a bet by conversion to a binary represenation.

        .. versionchanged: 0.3.0
            Changed definition of space

    balance : float
        The current balance of the environment.

    STARTING_BANK : int, default=10
        The starting bank / balance for the environment.
    """

    metadata = {'render.modes': ['human']}
    STARTING_BANK = 10

    def __init__(self, odds, odds_column_names, results=None):
        """Initializes a new environment

        Parameters
        ----------
        odds: dataframe of shape (n_games, n_odds)
            A list of games, with their betting odds.
        odds_column_names: list of str
            A list of column names with length == n_odds.
        results: list of int, default=None
            A list of the results, where results[i] is the outcome of odds[i].
        """

        super().__init__()
        self._odds = odds.copy()
        self._results = results
        self._odds_columns_names = odds_column_names
        self._verbose_actions = {act: [verbose_act] for verbose_act, act in zip(list(powerset(odds_column_names)),
                                                                                list(range(2 ** odds.shape[1])))}
        self.observation_space = gym.spaces.Box(low=1., high=float('Inf'), shape=(1, odds.shape[1]))
        self.action_space = gym.spaces.Discrete(2 ** odds.shape[1])
        self.balance = self.STARTING_BANK
        self.current_step = 0
        self.single_bet_size = 1

    def get_odds(self):
        """Returns the odds for the current step.

        Returns
        -------
        odds : dataframe of shape (1, n_odds)
            The odds for the current step.
        """
        return DataFrame([self._odds[self.current_step]])

    def get_bet(self, action):
        """Returns the betting matrix for the action provided.

        Parameters
        ----------
        action : int
            An action provided by the agent.

        Returns
        -------
        bet : array of shape (1, n_odds)
            The betting matrix, where each outcome specified in the action
            has a value of 1 and 0 otherwise.
        """
        verbose_actions = self._verbose_actions[action]
        bet = numpy.array([[int(name in verbose_action) for name in self._odds_columns_names]
                           for verbose_action in verbose_actions])
        return bet

    def step(self, action):
        """Run one timestep of the environment's dynamics. When end of episode is reached,
        you are responsible for calling reset() to reset this environment's state.

        Accepts an action and returns a tuple (observation, reward, done, info).

        Parameters
        ----------
        action : int
            An action provided by the agent.

        Returns
        -------
        observation : dataframe of shape (1, n_odds)
            The agent's observation of the current environment
        reward : float
            The amount of reward returned after previous action
        done : bool
            Whether the episode has ended, in which case further step() calls will return undefined results
        info : dict
            Contains auxiliary diagnostic information (helpful for debugging, and sometimes learning)

        """
        odds = self.get_odds()
        reward = 0
        done = False
        single_bet_size = self.single_bet_size
        info = self.create_info(action)
        if self.balance < 1:  # no more money :-(
            done = True
        else:
            bet = self.get_bet(action)
            if self.legal_bet(bet):  # making sure agent has enough money for the bet
                results = self.get_results()
                reward = ((bet * results * odds).values.sum() * single_bet_size) - \
                    (numpy.count_nonzero(bet) * single_bet_size)
                self.balance += reward
                info.update({'results': results.argmax()})
                self.current_step += 1
                if self.finish():
                    done = True
                    self.current_step = 0
            else:
                reward = -numpy.inf
        return self.get_odds(), reward, done, info

    def reset(self):
        """Resets the state of the environment and returns an initial observation.

        Returns
        -------
        observation : dataframe of shape (1, n_odds)
            the initial observation.
        """
        self.balance = self.STARTING_BANK
        self.current_step = 0
        return self.get_odds()

    def render(self, mode='human'):
        """Outputs the current balance and the current step.

        Returns
        -------
        msg : str
            A string with the current balance and the current step.
        """
        return 'Current balance at step {}: {}'.format(self.current_step, self.balance)

    def finish(self):
        """Checks if the episode has reached an end.

        The episode has reached an end if there are no more games to bet.

        Returns
        -------
        finish : bool
            True if the current_step is equal to n_games, False otherwise
        """
        return self.current_step == self._odds.shape[0]  # no more games left to bet

    def get_results(self):
        """Returns the results matrix for the current step.

        Returns
        -------
        result : array of shape (1, n_odds)
            The result matrix, where the index of the outcome that happened
            value is 1 and the rest of the indexes values are 0.
        """
        result = numpy.zeros(shape=self.observation_space.shape)
        result[numpy.arange(result.shape[0], dtype=numpy.int32),
               numpy.array([self._results[self.current_step]], dtype=numpy.int32)] = 1
        return result

    def legal_bet(self, bet):
        """Checks if the bet is legal.

        Checks that the bet does not exceed the current balance.

        Parameters
        ----------
        bet : array of shape (1, n_odds)
            The bet to check.

        Returns
        -------
        legal : bool
            True if the bet is legal, False otherwise.
        """
        return numpy.count_nonzero(bet) * self.single_bet_size <= self.balance

    def create_info(self, action):
        """Creates the info dictionary for the given action.

        The info dictionary holds the following information:
            * the verbose action
            * the current step
            * the balance at the start of the current step
            * the relevant odds for the current step
            * the bet size for a single outcome

        Parameters
        ----------
        action : int
            An action provided by the agent.

        Returns
        -------
        info : dict
            The info dictionary.
        """
        return {'action': self._verbose_actions[action], 'current_step': self.current_step,
                'starting_balance': self.balance, 'odds': self.get_odds(),
                'single_bet_size': self.single_bet_size}



import gym
import numpy
from pandas import DataFrame
from .base import BaseOddsEnv


class DailyOddsEnv(BaseOddsEnv):
    """Base class for sports betting environments multiple games with a fixed
    bet size.

    Creates an OpenAI Gym environment that supports betting a fixed amount (1)
    on a single outcome but for multiple games.

    .. versionadded:: 0.3.0

    Parameters
    ----------
    observation_space : gym.spaces.Box
        The observation space for the environment.
        The observation space shape is (M, N) where N is the number of possible
        outcomes for the game and M is the maximum number of games in a single
        day.

    action_space : gym.spaces.Box
        The action space for the environment.
        The action space shape is (M,), a list of numbers in [0, 2 ** N), represneting
        on what outcomes to place a bet by conversion to a binary represenation,
        where actions[i] is the action for odds[i].

    balance : float
        The current balance of the environment.

    STARTING_BANK : int, default=10
        The starting bank / balance for the environment.

    days : array, dtype=np.datetime64
        Sorted array of the days.

    """

    def __init__(self, odds, odds_column_names, results=None):
        """Initializes a new environment.

        We initialize the days array (because this environment doesn't iterate
        the games a single game at a time, but by a group of games that happened
        in the same date) and calculate the day with the most games in it so we
        that the observation & action spaces will be defined correctly.

        Parameters
        ----------
        odds: dataframe of shape (n_games, n_odds + 1)
            A list of games, with their betting odds and the date in which
            the game occures.

            .. warning::
                Please note that the environment expects the date column
                in the dataframe to be named 'date' (lower cased).

        odds_column_names: list of str
            A list of column names with length == n_odds.

        results: list of int, default=None
            A list of the results, where results[i] is the outcome of odds[i].
        """
        super().__init__(odds.drop('date', 'columns'), odds_column_names, results)
        self._odds_with_dates = odds.copy()
        self.days = odds['date'].unique()
        self.days.sort()
        max_number_of_games = odds.set_index('date').groupby(by='date').size().max()
        self.observation_space = gym.spaces.Box(low=1., high=float('Inf'),
                                                shape=(max_number_of_games, self._odds.shape[1]))
        self.action_space = gym.spaces.Box(low=0,
                                           high=2 ** self._odds.shape[1] - 0.01,
                                           shape=(max_number_of_games,))

    def get_odds(self):
        """Returns the odds for the current step.

        Returns
        -------
        odds : dataframe of shape (max_games, n_odds)
            The odds for the current step.
            If in the current step there were less then max_games, the odds
            dataframe is appended with (max_games - current_games) zeroed rows
            (rows with only 0).
        """
        current_day = self.days[self.current_step]
        current_odds = self._odds.iloc[self._odds_with_dates[self._odds_with_dates['date'] == current_day].index]
        filler_odds = DataFrame(numpy.zeros(numpy.array([*self.observation_space.shape]) -
                                            numpy.array([current_odds.shape[0], 0])),
                                columns=self._odds_columns_names)
        return current_odds.append(filler_odds, ignore_index=True)

    def get_bet(self, action):
        """Returns the betting matrix for the action provided.

        Parameters
        ----------
        action : array of shape (max_games, )
            An action provided by the agent.

        Returns
        -------
        bet : array of shape (max_games, n_odds)
            The betting matrix, where for each row, each outcome specified in
            action[row] has a value of 1 and 0 otherwise.
        """
        full_actions = numpy.zeros([*self.observation_space.shape])
        actions = numpy.concatenate([super(DailyOddsEnv, self).get_bet(numpy.floor(part_action))
                                     for part_action in action])
        full_actions[numpy.arange(actions.shape[0])] = actions
        return full_actions

    def get_results(self):
        """Returns the results matrix for the current step.

        Returns
        -------
        results : array of shape (max_games, n_odds)
            The results matrix, where for each row (single game),
            the index of the outcome that happened value is 1 and the rest of
            the indexes values are 0.
            If in the current step there were less then max_games, the result
            matrix is appended with (max_games - current_games) zeroed rows
            (rows with only 0).
        """
        current_day = self.days[self.current_step]
        index = self._odds_with_dates[self._odds_with_dates['date'] == current_day].index
        current_results = self._results.iloc[index]
        results = numpy.zeros(shape=(current_results.shape[0], self._odds.shape[1]))
        results[numpy.arange(results.shape[0]), current_results.values] = 1
        filler_results = numpy.zeros(numpy.array([*self.observation_space.shape]) -
                                     numpy.array([current_results.shape[0], 0]))
        return numpy.concatenate([results, filler_results])

    def finish(self):
        """Checks if the episode has reached an end.

        The episode has reached an end if there are no more days left.

        Returns
        -------
        finish : bool
            True if the current_step is equal to len(self.days), False otherwise
        """
        return self.current_step == self.days.shape[0]  # no more days left to bet

    def create_info(self, action):
        """Creates the info dictionary for the given action.

        The info dictionary holds the following information:
            * the verbose actions
            * the current step
            * the balance at the start of the current step
            * the relevant odds for the current step
            * the bet size for a single outcome

        Parameters
        ----------
        action : array of shape (max_games,)
            An action provided by the agent.

        Returns
        -------
        info : dict
            The info dictionary.
        """
        return {'action': [self._verbose_actions[act] for act in numpy.floor(action)],
                'current_step': self.current_step,
                'starting_balance': self.balance,
                'odds': self.get_odds(),
                'single_bet_size': self.single_bet_size}



import pytest
import numpy
from gym.spaces import Box


def test_attributes(three_way_percentage_env):
    assert three_way_percentage_env.observation_space == Box(low=1., high=float('Inf'), shape=(1, 3))
    assert three_way_percentage_env.action_space == Box(low=numpy.array([0., 0.01]),
                                                        high=numpy.array([2 ** 3 - 0.01, 1 / 3]))
    assert numpy.array_equal(three_way_percentage_env.teams, numpy.array([['FCB', 'PSG'], ['MCB', 'MTA']]))


@pytest.mark.parametrize("form,percent,expected_reward", [(0, 0.1, 0), (1, 0.1, -1), (2, 0.1, 1), (3, 0.1, -1)])
def test_step(three_way_percentage_env, form, percent, expected_reward):
    odds, reward, done, _ = three_way_percentage_env.step((form, percent))
    assert reward == expected_reward
    assert not done
    assert three_way_percentage_env.current_step == 1



import pytest
import numpy
from gym.spaces import Box


def test_attributes(three_way_env):
    assert three_way_env.action_space.n == 2 ** 3
    assert three_way_env.observation_space == Box(low=1., high=float('Inf'), shape=(1, 3))
    assert three_way_env.STARTING_BANK == 10
    assert three_way_env.balance == three_way_env.STARTING_BANK
    assert three_way_env.current_step == 0
    assert three_way_env.single_bet_size == 1
    assert numpy.array_equal(three_way_env.teams, numpy.array([['FCB', 'PSG'], ['MCB', 'MTA']]))


@pytest.mark.parametrize("action,expected_reward", [(0, 0), (1, -1), (2, 1), (3, -1), (4, 0),
                                                    (5, -2), (6, 0), (7, -1)])
def test_step(three_way_env, action, expected_reward):
    odds, reward, done, _ = three_way_env.step(action)
    assert reward == expected_reward
    assert not done
    assert three_way_env.current_step == 1


def test_multiple_steps(three_way_env):
    odds, reward, done, _ = three_way_env.step(3)
    assert reward == -1
    assert three_way_env.balance == three_way_env.STARTING_BANK - 1
    assert not done
    assert three_way_env.current_step == 1
    odds, reward, done, _ = three_way_env.step(1)
    assert reward == 3
    assert three_way_env.balance == three_way_env.STARTING_BANK - 1 + 3
    assert done



import pytest
from gym.spaces import Box
from pandas import DataFrame
from datetime import datetime, timedelta
import numpy
from numpy import array, zeros


def test_attributes(daily_bets_env):
    assert daily_bets_env.action_space == Box(low=0, high=2 ** 2 - 0.01, shape=(2,))
    assert daily_bets_env.observation_space == Box(low=1., high=float('Inf'), shape=(2, 2))
    assert daily_bets_env.STARTING_BANK == 10
    assert daily_bets_env.balance == daily_bets_env.STARTING_BANK
    assert daily_bets_env.current_step == 0
    assert daily_bets_env.single_bet_size == 1
    assert len(daily_bets_env.days) == 2
    assert daily_bets_env.days[0] == datetime.today().date() - timedelta(days=1)
    assert daily_bets_env.days[1] == datetime.today().date()


def test_get_odds(daily_bets_env):
    odds = daily_bets_env.get_odds()
    assert odds.equals(DataFrame([{'w': 1, 'l': 2}, {'w': 4, 'l': 3}], dtype=numpy.float64))


@pytest.mark.parametrize("actions,bets", [(array([0, 0]), array([[0, 0], [0, 0]])),
                                          (array([0, 1]), array([[0, 0], [1, 0]])),
                                          (array([0, 2]), array([[0, 0], [0, 1]])),
                                          (array([0, 3]), array([[0, 0], [1, 1]])),
                                          (array([1, 0]), array([[1, 0], [0, 0]])),
                                          (array([1, 1]), array([[1, 0], [1, 0]])),
                                          (array([1, 2]), array([[1, 0], [0, 1]])),
                                          (array([1, 3]), array([[1, 0], [1, 1]])),
                                          (array([2, 0]), array([[0, 1], [0, 0]])),
                                          (array([2, 1]), array([[0, 1], [1, 0]])),
                                          (array([2, 2]), array([[0, 1], [0, 1]])),
                                          (array([2, 3]), array([[0, 1], [1, 1]])),
                                          (array([3, 0]), array([[1, 1], [0, 0]])),
                                          (array([3, 1]), array([[1, 1], [1, 0]])),
                                          (array([3, 2]), array([[1, 1], [0, 1]])),
                                          (array([3, 3]), array([[1, 1], [1, 1]])),
                                          ])
def test_get_bet(daily_bets_env, actions, bets):
    assert numpy.array_equal(daily_bets_env.get_bet(actions), bets)


def test_attributes_of_non_uniform(daily_bets_env_non_uniform):
    assert daily_bets_env_non_uniform.action_space == Box(low=0, high=2 ** 2 - 0.01, shape=(3,))
    assert daily_bets_env_non_uniform.observation_space == Box(low=1., high=float('Inf'), shape=(3, 2))
    assert len(daily_bets_env_non_uniform.days) == 3
    assert daily_bets_env_non_uniform.days[0] == datetime.today().date() - timedelta(days=2)
    assert daily_bets_env_non_uniform.days[1] == datetime.today().date() - timedelta(days=1)
    assert daily_bets_env_non_uniform.days[2] == datetime.today().date()


@pytest.mark.parametrize("current_step_value,excpected_odds",
                         [(0, DataFrame([{'w': 1, 'l': 2}, {'w': 0, 'l': 0}, {'w': 0, 'l': 0}],
                                        dtype=numpy.float64)),
                          (1, DataFrame([{'w': 4, 'l': 3}, {'w': 0, 'l': 0}, {'w': 0, 'l': 0}],
                                        dtype=numpy.float64)),
                          (2, DataFrame([{'w': 4, 'l': 3}, {'w': 4, 'l': 3}, {'w': 5, 'l': 4}],
                                        dtype=numpy.float64))])
def test_get_odds_non_uniform(daily_bets_env_non_uniform, current_step_value, excpected_odds):
    daily_bets_env_non_uniform.current_step = current_step_value
    odds = daily_bets_env_non_uniform.get_odds()
    assert odds.equals(excpected_odds)


@pytest.mark.parametrize("actions,bets", [(array([0]), zeros([3, 2])),
                                          (array([0] * 2), zeros([3, 2])),
                                          (array([0] * 3), zeros([3, 2])),
                                          (array([1]), array([[1, 0], [0, 0], [0, 0]])),
                                          (array([1] * 2), array([[1, 0], [1, 0], [0, 0]])),
                                          (array([1] * 3), array([[1, 0], [1, 0], [1, 0]])),
                                          (array([2]), array([[0, 1], [0, 0], [0, 0]])),
                                          (array([2] * 2), array([[0, 1], [0, 1], [0, 0]])),
                                          (array([2] * 3), array([[0, 1], [0, 1], [0, 1]])),
                                          (array([3]), array([[1, 1], [0, 0], [0, 0]])),
                                          (array([3] * 2), array([[1, 1], [1, 1], [0, 0]])),
                                          (array([3] * 3), array([[1, 1], [1, 1], [1, 1]])),
                                          (array([0, 1]), array([[0, 0], [1, 0], [0, 0]])),
                                          (array([0, 2]), array([[0, 0], [0, 1], [0, 0]])),
                                          (array([0, 3]), array([[0, 0], [1, 1], [0, 0]])),
                                          (array([1, 0]), array([[1, 0], [0, 0], [0, 0]])),
                                          (array([1, 2]), array([[1, 0], [0, 1], [0, 0]])),
                                          (array([1, 3]), array([[1, 0], [1, 1], [0, 0]])),
                                          (array([2, 0]), array([[0, 1], [0, 0], [0, 0]])),
                                          (array([2, 1]), array([[0, 1], [1, 0], [0, 0]])),
                                          (array([2, 3]), array([[0, 1], [1, 1], [0, 0]])),
                                          (array([3, 0]), array([[1, 1], [0, 0], [0, 0]])),
                                          (array([3, 1]), array([[1, 1], [1, 0], [0, 0]])),
                                          (array([3, 2]), array([[1, 1], [0, 1], [0, 0]])),
                                          (array([0, 0, 1]), array([[0, 0], [0, 0], [1, 0]])),
                                          (array([0, 0, 2]), array([[0, 0], [0, 0], [0, 1]])),
                                          (array([0, 0, 3]), array([[0, 0], [0, 0], [1, 1]])),
                                          (array([0, 1, 0]), array([[0, 0], [1, 0], [0, 0]])),
                                          (array([0, 2, 0]), array([[0, 0], [0, 1], [0, 0]])),
                                          (array([0, 3, 0]), array([[0, 0], [1, 1], [0, 0]])),
                                          (array([1, 0, 0]), array([[1, 0], [0, 0], [0, 0]])),
                                          (array([2, 0, 0]), array([[0, 1], [0, 0], [0, 0]])),
                                          (array([3, 0, 0]), array([[1, 1], [0, 0], [0, 0]])),
                                          (array([0, 1, 1]), array([[0, 0], [1, 0], [1, 0]])),
                                          (array([0, 1, 2]), array([[0, 0], [1, 0], [0, 1]])),
                                          (array([0, 1, 3]), array([[0, 0], [1, 0], [1, 1]])),
                                          (array([0, 2, 1]), array([[0, 0], [0, 1], [1, 0]])),
                                          (array([0, 2, 2]), array([[0, 0], [0, 1], [0, 1]])),
                                          (array([0, 2, 3]), array([[0, 0], [0, 1], [1, 1]])),
                                          (array([0, 3, 1]), array([[0, 0], [1, 1], [1, 0]])),
                                          (array([0, 3, 2]), array([[0, 0], [1, 1], [0, 1]])),
                                          (array([0, 3, 3]), array([[0, 0], [1, 1], [1, 1]])),
                                          (array([1, 0, 1]), array([[1, 0], [0, 0], [1, 0]])),
                                          (array([1, 0, 2]), array([[1, 0], [0, 0], [0, 1]])),
                                          (array([1, 0, 3]), array([[1, 0], [0, 0], [1, 1]])),
                                          (array([1, 1, 0]), array([[1, 0], [1, 0], [0, 0]])),
                                          (array([1, 1, 2]), array([[1, 0], [1, 0], [0, 1]])),
                                          (array([1, 1, 3]), array([[1, 0], [1, 0], [1, 1]])),
                                          (array([1, 2, 0]), array([[1, 0], [0, 1], [0, 0]])),
                                          (array([1, 2, 1]), array([[1, 0], [0, 1], [1, 0]])),
                                          (array([1, 2, 2]), array([[1, 0], [0, 1], [0, 1]])),
                                          (array([1, 2, 3]), array([[1, 0], [0, 1], [1, 1]])),
                                          ])
def test_get_bet_non_uniform(daily_bets_env_non_uniform, actions, bets):
    assert numpy.array_equal(daily_bets_env_non_uniform.get_bet(actions), bets)


@pytest.mark.parametrize("current_step_value,excpected_results",
                         [(0, array([[0, 1], [0, 0], [0, 0]], dtype=numpy.float64)),
                          (1, array([[1, 0], [0, 0], [0, 0]], dtype=numpy.float64)),
                          (2, array([[1, 0], [1, 0], [1, 0]], dtype=numpy.float64))])
def test_get_results_non_uniform(daily_bets_env_non_uniform, current_step_value, excpected_results):
    daily_bets_env_non_uniform.current_step = current_step_value
    results = daily_bets_env_non_uniform.get_results()
    assert numpy.array_equal(results, excpected_results)


@pytest.mark.parametrize("current_step_value,action,excpected_reward,finished",
                         [(0, array([0, 0]), 0, False),
                          (1, array([0, 0]), 0, True),
                          (0, array([1, 1]), 2, False),
                          (1, array([1, 1]), 6, True),
                          (0, array([1, 2]), -2, False),
                          (1, array([1, 2]), 2, True),
                          (0, array([2, 2]), 0, False),
                          (1, array([2, 2]), -2, True),
                          (0, array([3, 3]), 2, False),
                          (1, array([3, 3]), 4, True)])
def test_step(daily_bets_env, current_step_value, action, excpected_reward, finished):
    daily_bets_env.current_step = current_step_value
    odds, reward, done, info = daily_bets_env.step(action)
    assert reward == excpected_reward
    assert done == finished


@pytest.mark.parametrize("current_step_value,action,excpected_reward,finished",
                         [(0, array([0] * 3), 0, False),
                          (1, array([0] * 3), 0, False),
                          (2, array([0] * 3), 0, True),
                          (0, array([1] * 3), -3, False),
                          (1, array([1] * 3), 1, False),
                          (2, array([1] * 3), 10, True),
                          (0, array([3] * 3), -4, False),
                          (1, array([3] * 3), -2, False),
                          (2, array([3] * 3), 7, True),
                          (0, array([1, 1]), -2, False),
                          (1, array([1, 2]), 2, False),
                          (2, array([1, 3]), 5, True),
                          (0, array([2, 1]), 0, False),
                          (1, array([2, 2]), -2, False),
                          (2, array([2, 3]), 1, True)
                          ])
def test_step_non_uniform(daily_bets_env_non_uniform, current_step_value, action, excpected_reward, finished):
    daily_bets_env_non_uniform.current_step = current_step_value
    odds, reward, done, info = daily_bets_env_non_uniform.step(action)
    assert reward == excpected_reward
    assert done == finished



"""This module contains the packaging routine for oddsgym"""

from setuptools import setup, find_packages
from bumpversion.cli import _load_configuration

setup(packages=find_packages(),
      install_requires=['gym', 'more-itertools', 'pandas'],
      version=_load_configuration('setup.cfg', None, {})[0]['bumpversion']['current_version'])
